#############################
# required varibles
#
# LFLAGS, LIBS - for link
# CPP_INCLUDES, CPP_FLAGS - for g++ compilation
# C_INCLUDES, C_FLAGS - for cc compilation
#############################
# TOOL CHAINS

export _COMMON_INCLUDED=yes

#generic tools (*nix style)
MKDIR = mkdir -p
CP = cp
RM = rm -f
CHMOD = chmod
TAR = tar -czf
UNTAR = tar -xzf
ECHO = @echo
link?=static
MTFLAGS=
ifeq ($(link),static)
MTFLAGS:=mt-s$(if $(DEBUG),d)
else ifeq ($(link),dynamic)
MTFLAGS:=mt$(if $(DEBUG),-d)
endif
export MTFLAGS

###############
# Common flags for all toolchains

CPP_FLAGS = $(if $(DEBUG), -g3 -O0 -gdwarf-2 -ggdb -D_DEBUG, -Os -fno-inline)


##############
# Flags for each toolchain

ifeq ($(TOOLCHAIN),gcc-linux-pi)

  # linux-pc tool chain
  TIME_ = /usr/bin/time
  
#TIME_LINE=$(TIME_) --format="compiled in (%U user, %S sys) '$<'" 
TIME_LINE=

  CC = $(TIME_LINE) gcc -c -march=armv7-a -marm -fPIC
  CPP = $(TIME_LINE) g++ -c -march=armv7-a -marm -fPIC #colorgcc -c
  CPP_ = $(TIME_LINE) g++ -c -march=armv7-a -marm -fPIC
  LD = $(TIME_LINE) g++
  AR = $(TIME_LINE) ar
  STRIP = $(TIME_LINE)  strip
  
  TARGET_OSTYPE=pi
  TARGET_OSTYPE_EXE_EXT=o
  TARGET_OSTYPE_DLL_EXT=so

else ifeq ($(TOOLCHAIN),gcc-linux-pc)

  # linux-pc tool chain
  TIME_ = /usr/bin/time
  
#TIME_LINE=$(TIME_) --format="compiled in (%U user, %S sys) '$<'" 
TIME_LINE=

  CC = $(TIME_LINE) gcc -c 
  CPP = $(TIME_LINE) g++ -c 
  CPP_ = $(TIME_LINE) g++ -c
  LD = $(TIME_LINE) gcc
  AR = $(TIME_LINE) ar
  STRIP = $(TIME_LINE)  strip
  
  TARGET_OSTYPE=pi
  TARGET_OSTYPE_EXE_EXT=o
  TARGET_OSTYPE_DLL_EXT=so

else ifneq ($(MAKECMDGOALS),clean)
   $(error Unknown TOOLCHAIN='$(TOOLCHAIN)'! Available toolchains : [gcc-linux-pc | gcc-linux-pi])
   
endif
  
 CC_VERSION?=$(shell $(CC) --version | grep -e '^[0-9a-zA-Z_-]*gcc (.*)' | sed -e 's/[0-9a-zA-Z_-]*gcc (.*) \([0-9]\)\.\([0-9]\).*/gcc\1\2/')

#############################
# GCC COMPILE/LINK defines

define common-gcc-compile
  @$(MKDIR) $(@D)
  @$(ECHO) "compiling '$<' ... "
  $(CPP) $(CPP_FLAGS) $(CPP_INCLUDES) -MMD -MP -MF"$(@:%.o=%.d)" -MT"$(@:%.o=%.d)" -o"$@" "$<"
endef

define common-g++-compile
  @$(MKDIR) -p $(@D)
  @$(ECHO) "compiling '$<' ... "
  $(CPP_) $(CPP_FLAGS) $(CPP_INCLUDES) -MMD -MP -MF"$(@:%.o=%.d)" -MT"$(@:%.o=%.d)" -o"$@" "$<"
endef

define common-cc-compile
  $(MKDIR) -p $(@D)
  @$(ECHO) -n "compiling '$<' ... "
  $(CC) $(C_FLAGS) $(CPP_FLAGS) $(CPP_INCLUDES) -MMD -MP -MF"$(@:%.o=%.d)" -MT"$(@:%.o=%.d)" -o"$@" "$<" 
endef

define common-gcc-link
  $(ECHO) "linking '$@' ..."
  $(LD) -o"$@" $+ $(LFLAGS) $(LIBS)
endef

define common-gcc-ar
  $(ECHO) "ar-ing '$@' ..."
  $(AR) -r "$@" $+ 
endef

