#include <commands.h>
#include <stdio.h>
#include <string.h>
#include <crc_ccitt.h>

int serializeRequest(uint8_t* buf, int bufSize, BasicCmd_Req* req) {
	int index = 0;
	*(buf + index) = req->m_ucCmdId; index++;
	switch (req->m_ucCmdId) {	
		case FW_UPGRADE_START_REQ:
		{
			FwUpgradeStartCmd_req* r = (FwUpgradeStartCmd_req*)req;
			*(buf + (index++)) = (uint8_t)(r->m_ulFwLen & 0xFF);
			*(buf + (index++)) = (uint8_t)(r->m_ulFwLen >> 8);
			*(buf + (index++)) = (uint8_t)(r->m_ulFwLen >> 16);
			*(buf + (index++)) = (uint8_t)(r->m_ulFwLen >> 24);
		}
		break;
		case FW_UPGRADE_DATA_REQ:
		{
			FwUpgradeDataCmd_req* r = (FwUpgradeDataCmd_req*)req;
			*(buf + (index++)) = (uint8_t)(r->m_ulSeqNo & 0xFF);
			*(buf + (index++)) = (uint8_t)(r->m_ulSeqNo >> 8);
			*(buf + (index++)) = (uint8_t)(r->m_ulSeqNo >> 16);
			*(buf + (index++)) = (uint8_t)(r->m_ulSeqNo >> 24);
			 	             
			*(buf + (index++)) = r->m_ucPduLen;
			
			for (int i = 0; i < r->m_ucPduLen; i++) {
				*(buf + (index++)) = r->bytes[i];
			}
		}
		break;
		case FW_UPGRADE_DONE_REQ:
		{		
			FwUpgradeDoneCmd_req* r = (FwUpgradeDoneCmd_req*)req;
			*(buf + (index++)) = (uint8_t)(r->m_ulBytesTransferred & 0xFF);
			*(buf + (index++)) = (uint8_t)(r->m_ulBytesTransferred >> 8);
			*(buf + (index++)) = (uint8_t)(r->m_ulBytesTransferred >> 16);
			*(buf + (index++)) = (uint8_t)(r->m_ulBytesTransferred >> 24);
			 	             
			*(buf + (index++)) = (uint8_t)(r->m_unFwVersion & 0xFF);
			*(buf + (index++)) = (uint8_t)(r->m_unFwVersion >> 8);

			*(buf + (index++)) = (uint8_t)(r->m_unFwCrc & 0xFF);
			*(buf + (index++)) = (uint8_t)(r->m_unFwCrc >> 8);
		}
		break;
		case RESET_PROCESSOR_REQ:
		{
		    //empty request
		}
		break;
		case CONF_RADIO_REQ:
		{
		    ConfRadio_req* r = (ConfRadio_req*)req;
		    for (int i = 0; i < 16; i++) {
			*(buf + (index++)) = (uint8_t)r->m_ucMode[i];
		    }
		}
		break;
		case START_LIVE_CAPTURE_REQ:
		{
		    StartLiveCapture_req* r = (StartLiveCapture_req*)req;
		    *(buf + (index++)) = r->m_ucMode;
		}
		break;
		case GET_VERSION_REQ:
		case DEBUGINFO_REQ:
		//TODO the rest
			break;
		default:
			break;
	}
	uint16_t crc = Crc_CcittCalculate(buf, index);
	*(buf + (index++)) = (uint8_t)(crc & 0xFF);
	*(buf + (index++)) = (uint8_t)(crc >> 8);
	
	return index;
}

int serializeResponse(uint8_t* buf, int bufSize, BasicCmd_Resp* rsp) {
	int index = 0;
	*(buf + index) = rsp->m_ucCmdId; index++;
	*(buf + index) = rsp->m_ucStatus; index++;
	switch (rsp->m_ucCmdId) {	
		case GET_VERSION_RSP:
		{		
			GetVersionCmd_resp* r = (GetVersionCmd_resp*)rsp;
			*(buf + (index++)) = (uint8_t)(r->m_unVersion & 0xFF);
			*(buf + (index++)) = (uint8_t)(r->m_unVersion >> 8);
		}
		break;
		case NACK_RSP:
		{
			Nack_resp* r = (Nack_resp*)rsp;
			*(buf + (index++)) = r->m_ucCode;
		}
		break;
		case RADIO_PACKET_RSP:
		{
			RadioPacket_resp* r = (RadioPacket_resp*)rsp;
			*(buf + (index++)) = (uint8_t)(r->m_ucFlags & 0xFF);
			
			*(buf + (index++)) = (uint8_t)(r->m_ullTimeStamp & 0xFF);
			*(buf + (index++)) = (uint8_t)(r->m_ullTimeStamp >> 8);
			*(buf + (index++)) = (uint8_t)(r->m_ullTimeStamp >> 16);
			*(buf + (index++)) = (uint8_t)(r->m_ullTimeStamp >> 24);			
			*(buf + (index++)) = (uint8_t)(r->m_ullTimeStamp >> 32);
			*(buf + (index++)) = (uint8_t)(r->m_ullTimeStamp >> 40);
			*(buf + (index++)) = (uint8_t)(r->m_ullTimeStamp >> 48);
			*(buf + (index++)) = (uint8_t)(r->m_ullTimeStamp >> 56);
			
			*(buf + (index++)) = (uint8_t)(r->m_ucChannel & 0xFF);
			*(buf + (index++)) = (uint8_t)(r->m_ucRsl & 0xFF);
			
			*(buf + (index++)) = (uint8_t)(r->m_ulGpsLat & 0xFF);
			*(buf + (index++)) = (uint8_t)(r->m_ulGpsLat >> 8);
			*(buf + (index++)) = (uint8_t)(r->m_ulGpsLat >> 16);
			*(buf + (index++)) = (uint8_t)(r->m_ulGpsLat >> 24);

			*(buf + (index++)) = (uint8_t)(r->m_ulGpsLong & 0xFF);
			*(buf + (index++)) = (uint8_t)(r->m_ulGpsLong >> 8);
			*(buf + (index++)) = (uint8_t)(r->m_ulGpsLong >> 16);
			*(buf + (index++)) = (uint8_t)(r->m_ulGpsLong >> 24);

			*(buf + (index++)) = (uint8_t)(r->m_ucPduLen & 0xFF);			
			for (int i = 0; i < r->m_ucPduLen; i++) {
				*(buf + (index++)) = (uint8_t)(r->m_ucPdu[i] & 0xFF);				
			}
			
			*(buf + (index++)) = (uint8_t)(r->m_ucExtraPduLen & 0xFF);			
			for (int i = 0; i < r->m_ucExtraPduLen; i++) {
				*(buf + (index++)) = (uint8_t)(r->extraPDU[i] & 0xFF);				
			}						
		}
		break;
		case FW_UPGRADE_START_RSP:
		case FW_UPGRADE_DATA_RSP:
		case FW_UPGRADE_DONE_RSP:
		case RESET_PROCESSOR_RSP:
		case CONF_RADIO_RSP:
		//TODO the rest
			break;
	
		default:
			break;
	}
	uint16_t crc = Crc_CcittCalculate(buf, index);
	*(buf + (index++)) = (uint8_t)(crc & 0xFF);
	*(buf + (index++)) = (uint8_t)(crc >> 8);

	return index;
}

int parseRequest(uint8_t* buf, int bufSize, void* cmdbuf) {

	//TODO
}
int parseResponse(uint8_t* buf, int bufSize, void* cmdbuf) {
	BasicCmd_Resp* rsp = (BasicCmd_Resp*)cmdbuf;
	int index = 0;
		
	rsp->m_ucCmdId = *(buf + (index++));	
	
	if (rsp->m_ucCmdId == RESET_PROCESSOR_RSP)	// does not contain CRC
	{
	    return index;
	}

	if (bufSize < 2)
		return -1;

	switch (rsp->m_ucCmdId) {
		case GET_VERSION_RSP:
		{
			if (bufSize < 4)
				return -1;
			
			rsp->m_ucStatus = *(buf + (index++));	
			
			GetVersionCmd_resp* r = (GetVersionCmd_resp*)rsp;
			r->m_unVersion = *(buf + (index++));
			r->m_unVersion |= (*(buf + (index++))) << 8;			
		}
		break;
		case NACK_RSP:
		{
			if (bufSize < 3)
				return -1;
			
			rsp->m_ucStatus = *(buf + (index++));	

			Nack_resp* r = (Nack_resp*)rsp;
			r->m_ucCode = *(buf + (index++));
		}
		break;
		case RADIO_PACKET_RSP:
		{
			RadioPacket_resp* r = (RadioPacket_resp*)rsp;
			r->m_ucFlags = *(buf + (index++));
			
			r->m_ullTimeStamp = *(buf + (index++));
			r->m_ullTimeStamp |= (uint64_t)*(buf + (index++)) << 8;
			r->m_ullTimeStamp |= (uint64_t)*(buf + (index++)) << 16;
			r->m_ullTimeStamp |= (uint64_t)*(buf + (index++)) << 24;
			r->m_ullTimeStamp |= (uint64_t)*(buf + (index++)) << 32;
			r->m_ullTimeStamp |= (uint64_t)*(buf + (index++)) << 40;
			r->m_ullTimeStamp |= (uint64_t)*(buf + (index++)) << 48;
			r->m_ullTimeStamp |= (uint64_t)*(buf + (index++)) << 56;

			r->m_ucChannel = *(buf + (index++));
			r->m_ucRsl = *(buf + (index++));
			
			r->m_ulGpsLat = *(buf + (index++));
			r->m_ulGpsLat |= (uint32_t)*(buf + (index++)) << 8;
			r->m_ulGpsLat |= (uint32_t)*(buf + (index++)) << 16;
			r->m_ulGpsLat |= (uint32_t)*(buf + (index++)) << 24;

			r->m_ulGpsLong = *(buf + (index++));
			r->m_ulGpsLong |= (uint32_t)*(buf + (index++)) << 8;
			r->m_ulGpsLong |= (uint32_t)*(buf + (index++)) << 16;
			r->m_ulGpsLong |= (uint32_t)*(buf + (index++)) << 24;

			r->m_ucPduLen = *(buf + (index++));		
			for (int i = 0; i < r->m_ucPduLen; i++) {
				r->m_ucPdu[i] = *(buf + (index++));				
			}
			
			//r->m_ucExtraPduLen = *(buf + (index++));			
			r->m_ucExtraPduLen = bufSize - index - 2;
			for (int i = 0; i < r->m_ucExtraPduLen; i++) {
				r->extraPDU[i] = *(buf + (index++));				
			}			
		}
		break;
		case DEBUGINFO_RSP:
		    index += 17;
		    rsp->m_ucStatus = *(buf + (index++));	
		break;
		case ED_PACKET_RSP:
		{
		    EDPacket_resp* r = (EDPacket_resp*)rsp;
		    r->m_ucFlags = *(buf + (index++));
			
		    r->m_ullTimeStamp = *(buf + (index++));
		    r->m_ullTimeStamp |= (uint64_t)*(buf + (index++)) << 8;
		    r->m_ullTimeStamp |= (uint64_t)*(buf + (index++)) << 16;
		    r->m_ullTimeStamp |= (uint64_t)*(buf + (index++)) << 24;
		    r->m_ullTimeStamp |= (uint64_t)*(buf + (index++)) << 32;
		    r->m_ullTimeStamp |= (uint64_t)*(buf + (index++)) << 40;
		    r->m_ullTimeStamp |= (uint64_t)*(buf + (index++)) << 48;
		    r->m_ullTimeStamp |= (uint64_t)*(buf + (index++)) << 56;

		    r->m_ucChannel = *(buf + (index++));
		    r->m_ucRsl = *(buf + (index++));
		    
		    r->m_ulGpsLat = *(buf + (index++));
		    r->m_ulGpsLat |= (uint32_t)*(buf + (index++)) << 8;
		    r->m_ulGpsLat |= (uint32_t)*(buf + (index++)) << 16;
		    r->m_ulGpsLat |= (uint32_t)*(buf + (index++)) << 24;
		    
		    r->m_ulGpsLong = *(buf + (index++));
		    r->m_ulGpsLong |= (uint32_t)*(buf + (index++)) << 8;
		    r->m_ulGpsLong |= (uint32_t)*(buf + (index++)) << 16;
		    r->m_ulGpsLong |= (uint32_t)*(buf + (index++)) << 24;
		}
		break;
		default:
			rsp->m_ucStatus = *(buf + (index++));	
		break;
	}
	uint16_t computedCRC = Crc_CcittCalculate(buf, index);
	uint16_t actualCRC = *(buf + index) + (((uint16_t)*(buf + index + 1)) << 8);
		

//	fprintf(stderr, " Computed CRC=%#04X, Actual CRC=%#04X\n", computedCRC, actualCRC);
	if (computedCRC == actualCRC) {
		return index;
	}
	
	fprintf(stderr, "CRC error\n");
	return -1;
}

int escapePacket(uint8_t* buf, int bufSize) {
	for (int i = 0; i < bufSize; i++) {
		switch (*(buf + i)) 
		{
			case STX:
			case ETX:
			case CHX:
				*(buf + i) = ~(*(buf + i));
				memmove(buf + i + 1, buf + i, bufSize - i);
				*(buf + i) = CHX;
				bufSize++;
			break;
			default:
			break;
		}
	}
	memmove(buf + 1, buf, bufSize);
	*(buf) = STX;
	*(buf + bufSize + 1) = ETX;
	
	return bufSize + 2;
}

int unescapePacket(uint8_t* buf, int bufSize) {
	for (int i = 1; i < bufSize; i++) {
		switch (*(buf + i)) 
		{
			case STX:
			case ETX:
			break;
			case CHX:
				memmove(buf + i, buf + i + 1, bufSize - i - 1);
				*(buf + i) = ~(*(buf + i));
				bufSize--;
			break;
			default:
			break;
		}
	}
	memmove(buf, buf + 1, bufSize - 2);
			
	return bufSize - 2;
}
