#include <stdint.h>

#define STX 0xA1
#define ETX 0xA3
#define CHX 0xA2

enum CMDID
{
	/* Request IDs start from 0 up to 100 */
	/* Requests for bootloader */
	FW_UPGRADE_START_REQ = 0,
	FW_UPGRADE_DATA_REQ = 1,
	FW_UPGRADE_DONE_REQ = 2,
	FW_UPGRADE_ABORT_REQ = 3,
	START_APPLICATION_REQ = 4,
	STATUS_REQ = 5,

	/* Requests for application*/
	GET_VERSION_REQ = 6,
	/* FW upgrade commands TBD */
	//	FW_UPGRADE				= 1,
	CONF_RADIO_REQ = 7,
	/* Start live capture commands TBD*/
	START_LIVE_CAPTURE_REQ = 8,
	STOP_LIVE_CAPTURE_REQ = 9,
	GET_SD_FILE_LIST_REQ = 10,
	OPEN_SD_FILE_LIST_ITEM_REQ = 11,
	ERASE_SD_FILE_LIST_ITEM_REQ = 12,
	ERASE_SD_CARD_REQ = 13,
	RESET_PROCESSOR_REQ = 14,
	DEBUGINFO_REQ = 15,

	// 	/* Requests for application*/
	// 	GET_VERSION_REQ				= 13,
	// 	/* FW upgrade commands TBD */
	// 	//	FW_UPGRADE				= 1,
	// 	CONF_RADIO_REQ				= 14,
	// 	/* Start live capture commands TBD*/
	// 	START_LIVE_CAPTURE_REQ		= 15,
	// 	STOP_LIVE_CAPTURE_REQ		= 16,
	// 	GET_SD_FILE_LIST_REQ		= 17,
	// 	OPEN_SD_FILE_LIST_ITEM_REQ	= 18,
	// 	ERASE_SD_FILE_LIST_ITEM_REQ	= 19,
	// 	ERASE_SD_CARD_REQ			= 20,
	//SET_ED_INTERVAL_REQ			= 21,


	/* Response IDs start from 100 */
	/* Responses for bootloader */
	FW_UPGRADE_START_RSP = 100,
	FW_UPGRADE_DATA_RSP = 101,
	FW_UPGRADE_DONE_RSP = 102,
	FW_UPGRADE_ABORT_RSP = 103,
	START_APPLICATION_RSP = 104,
	STATUS_RSP = 105,

	/* Responses for applicaion */
	GET_VERSION_RSP = 107,
	/* FW upgrade commands TBD */
	FW_UPGRADE_RSP = 108,
	CONF_RADIO_RSP = 109,
	/* Start live capture commands TBD*/
	START_LIVE_CAPTURE_RSP = 110,
	STOP_LIVE_CAPTURE_RSP = 111,
	GET_SD_FILE_LIST_RSP = 112,
	/* TBD */
	// 	OPEN_SD_FILE_LIST_ITEM	= 6,
	// 	ERASE_SD_FILE_LIST_ITEM	= 7,
	// 	ERASE_SD_CARD			= 8,
	RADIO_PACKET_RSP = 113,
	NACK_RSP = 114,
	RESET_PROCESSOR_RSP = 115,
	ED_PACKET_RSP = 116,
	DEBUGINFO_RSP = 117
	//SET_ED_INTERVAL_RSP			= 116
};

typedef uint8_t cmdbuf_t[512];

typedef struct {
	uint8_t m_ucCmdId;
} BasicCmd_Req;

typedef struct {
	uint8_t m_ucCmdId;
	uint8_t	m_ucStatus;
}BasicCmd_Resp;

typedef BasicCmd_Req ResetProcessorCmd_req;
typedef BasicCmd_Resp ResetProcessorCmd_resp;

typedef BasicCmd_Req GetVersionCmd_req;
typedef struct {
	BasicCmd_Resp rsp;
	uint16_t m_unVersion;
} GetVersionCmd_resp;

typedef struct {
	BasicCmd_Req req;
	uint32_t m_ulFwLen;
} FwUpgradeStartCmd_req;          

typedef BasicCmd_Resp FwUpgradeStartCmd_resp;

typedef struct {
	BasicCmd_Req req;
	uint32_t  m_ulSeqNo;
	uint8_t	m_ucPduLen;
	uint8_t bytes[128];
} FwUpgradeDataCmd_req;

typedef BasicCmd_Resp FwUpgradeDataCmd_resp;

typedef struct {
	BasicCmd_Req req;
	uint32_t m_ulBytesTransferred;
	uint16_t m_unFwVersion;
	uint16_t m_unFwCrc;			
} FwUpgradeDoneCmd_req;

typedef BasicCmd_Resp FwUpgradeDoneCmd_resp;

typedef BasicCmd_Req FwUpgradeAbortCmd_req;
typedef BasicCmd_Resp FwUpgradeAbortCmd_resp;

typedef BasicCmd_Req StartApplicationCmd_req;
typedef BasicCmd_Resp StartApplicationCmd_resp;

typedef BasicCmd_Req StatusCmd_req;
typedef BasicCmd_Resp StatusCmd_resp;

typedef struct {
	BasicCmd_Resp rsp;
	uint8_t m_ucCode;
} Nack_resp;

typedef struct {
	BasicCmd_Resp rsp;
	uint8_t m_ucFlags;
	uint64_t m_ullTimeStamp;
	uint8_t m_ucChannel;
	uint8_t m_ucRsl;
	uint32_t m_ulGpsLat;
	uint32_t m_ulGpsLong;
	uint8_t m_ucPduLen;
	uint8_t m_ucPdu[256];
	uint8_t decFlags;	//TODO check why this is here
	uint8_t m_ucExtraPduLen;
	uint8_t extraPDU[128];
} RadioPacket_resp;

typedef struct {
	BasicCmd_Resp rsp;
	uint8_t m_ucFlags;
	uint8_t m_ucChannel;
	uint8_t m_ucRsl;
	uint32_t m_ulGpsLat;
	uint32_t m_ulGpsLong;
	uint64_t m_ullTimeStamp;
} EDPacket_resp;

typedef struct {
    uint8_t m_ucCmdId;
    uint8_t m_ucMode[16];
} ConfRadio_req;

typedef BasicCmd_Resp ConfRadio_rsp;

typedef struct {
    uint8_t m_ucCmdId;
    uint8_t m_ucMode;
} StartLiveCapture_req;

typedef BasicCmd_Resp StartLiveCapture_rsp;

typedef BasicCmd_Req DebugInfo_req;

typedef struct {
	BasicCmd_Resp rsp;
	uint8_t m_ucFlags;
	uint8_t m_radioStatus[16];
} DebugInfo_rsp;

int serializeRequest(uint8_t* buf, int bufSize, BasicCmd_Req* req);
int serializeResponse(uint8_t* buf, int bufSize, BasicCmd_Resp* rsp);

int parseRequest(uint8_t* buf, int bufSize, void* cmdbuf);
int parseResponse(uint8_t* buf, int bufSize, void* cmdbuf);

int escapePacket(uint8_t* buf, int bufSize);
int unescapePacket(uint8_t* buf, int bufSize);