#include <stdio.h>
#include <stdint.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <ftd2xx.h>

#define MAX_DEVICES 5

FT_HANDLE	ftHandle;
FT_DEVICE_LIST_INFO_NODE *devInfo; 
DWORD numDevs; 
FT_STATUS	ftStatus;

int ftdi_openDevice(int index) 
{
	DWORD	dwRxSize = 0;
	int	iNumDevs = 0;
	int	i, j;
	int	iDevicesOpen;	
	
	ftStatus = FT_CreateDeviceInfoList(&numDevs); 
	if (ftStatus == FT_OK) { 
	   fprintf(stderr, "Number of connected devices is %d\n", numDevs); 
	} 
	devInfo = (FT_DEVICE_LIST_INFO_NODE*)malloc(sizeof(FT_DEVICE_LIST_INFO_NODE)*numDevs); 

	ftStatus = FT_GetDeviceInfoList(devInfo,&numDevs); 
	
	if(ftStatus != FT_OK) {
		fprintf(stderr, "Error: FT_GetDeviceInfoList(%d)\n", (int)ftStatus);
		free(devInfo);
		return -1;
	}

	for(i = 0; i < numDevs; i++) {
		fprintf(stderr, "Device %d Serial Number - %s - Description=\"%s\"\n", i, devInfo[i].SerialNumber, devInfo[i].Description);
	}

	if (index >= numDevs) {
		fprintf(stderr, "Device with index %d not found. Found a number of %d devices.", index, numDevs);
		free(devInfo);
		return -1;
	}			
	/* Setup */
	if((ftStatus = FT_OpenEx(devInfo[index].SerialNumber, FT_OPEN_BY_SERIAL_NUMBER, &ftHandle)) != FT_OK){
		/* 
			This can fail if the ftdi_sio driver is loaded
			use lsmod to check this and rmmod ftdi_sio to remove
			also rmmod usbserial
		*/
		fprintf(stderr, "Error FT_OpenEx(%d), device %d\n", (int)ftStatus, index);
		fprintf(stderr, "Use lsmod to check if ftdi_sio (and usbserial) are present.\n");
		fprintf(stderr, "If so, unload them using rmmod, as they conflict with ftd2xx.\n");
		free(devInfo);
		return -1;
	}

	fprintf(stderr, "Opened device %s\n", devInfo[index].SerialNumber);

	iDevicesOpen++;
//	if((ftStatus = FT_SetBaudRate(ftHandle, 9600)) != FT_OK) {
	if((ftStatus = FT_SetBaudRate(ftHandle, 3000000)) != FT_OK) {
		fprintf(stderr, "Error FT_SetBaudRate(%d), cBufLD[i] = %s\n", (int)ftStatus, devInfo[index].SerialNumber);
		free(devInfo);
		return -1;
	}
	
	if((ftStatus = FT_SetTimeouts(ftHandle, 3000, 3000)) != FT_OK) {
		fprintf(stderr, "Error FT_SetTimeouts(%d), cBufLD[i] = %s\n", (int)ftStatus, devInfo[index].SerialNumber);
		free(devInfo);
		return -1;
	}
	
//	if((ftStatus = FT_SetFlowControl(ftHandle, FT_FLOW_XON_XOFF, 0x11, 0x13)) != FT_OK) {
//	if((ftStatus = FT_SetFlowControl(ftHandle, FT_FLOW_RTS_CTS, 0x11, 0x13)) != FT_OK) {
	if((ftStatus = FT_SetFlowControl(ftHandle, FT_FLOW_NONE, 0, 0)) != FT_OK) {
		fprintf(stderr, "Error FT_SetFlowControl(%d), cBufLD[i] = %s\n", (int)ftStatus, devInfo[index].SerialNumber);
		free(devInfo);
		return -1;
	}
	
	if((ftStatus = FT_SetDataCharacteristics(ftHandle, FT_BITS_8, FT_STOP_BITS_1, FT_PARITY_NONE)) != FT_OK) {
		fprintf(stderr, "Error FT_SetFlowControl(%d), cBufLD[i] = %s\n", (int)ftStatus, devInfo[index].SerialNumber);
		free(devInfo);
		return -1;
	}
/*
	if ((ftStatus = FT_SetLatencyTimer(ftHandle, 255)) != FT_OK) {
		fprintf(stderr, "Error FT_SetLatencyTimer(%d), cBufLD[i] = %s\n", (int)ftStatus, devInfo[index].SerialNumber);
		free(devInfo);
		return -1;
	}
	
	if ((ftStatus = FT_SetUSBParameters(ftHandle, 16384, 16384)) != FT_OK) {
		fprintf(stderr, "Error FT_SetLatencyTimer(%d), cBufLD[i] = %s\n", (int)ftStatus, devInfo[index].SerialNumber);
		free(devInfo);
		return -1;
	}
*/
//	FT_SetBitMode(ftHandle, 0xFF, 0x0);
	
	

	free(devInfo);
	return 0;
}

int ftdi_writeBytes(char* cBufWrite, uint16_t bufSize) {
	DWORD 	dwBytesWritten;
	/* Write */
//	for (int i = 0; i < bufSize; i++) {
//	    fprintf(stderr, "%02X", (uint8_t)*(cBufWrite + i));
//	    if (i == bufSize - 1)
//		fprintf(stderr, "\n");
//	    else
//		fprintf(stderr, ", ");
//	}

	ftStatus = FT_Write(ftHandle, cBufWrite, bufSize, &dwBytesWritten);
	if (ftStatus != FT_OK) {
		fprintf(stderr, "Error FT_Write(%d)\n", (int)ftStatus);
		return -1;
	}
	if (dwBytesWritten != (DWORD)bufSize) {
		fprintf(stderr, "FT_Write only wrote %d (of %d) bytes\n", 
			   (int)dwBytesWritten, 
			   bufSize);
	}	

	return dwBytesWritten;	
}

int ftdi_readBytes(char *pcBufRead, uint16_t bufSize) {
	/* Read */
	DWORD dwRxSize = 0;
	DWORD dwBytesRead = 0;
	DWORD totalBytes = 0;

	do {
	ftStatus = FT_GetQueueStatus(ftHandle, &dwRxSize);
	if(ftStatus == FT_OK) {
		if (dwRxSize > 0) {
			ftStatus = FT_Read(ftHandle, pcBufRead + totalBytes, (dwRxSize > (bufSize - totalBytes) ? (bufSize - totalBytes) : dwRxSize), &dwBytesRead);
			if (ftStatus != FT_OK) {
				fprintf(stderr, "Error FT_Read(%d)\n", (int)ftStatus);
				return -1;
			}
//	fprintf(stderr," Rx:");
//	for (int i = 0; i < dwBytesRead; i++) {
//	    fprintf(stderr, "%02X", (uint8_t)*(pcBufRead + i));
//	    if (i == dwBytesRead - 1)
//		fprintf(stderr, "\n");
//	    else
//		fprintf(stderr, ", ");
//	}
		}
	    }
	    else {
		fprintf(stderr, "Error FT_GetQueueStatus(%d)\n", (int)ftStatus);	
		exit(0);
	    }

	    totalBytes += dwBytesRead;
	} while (dwRxSize > 0);
	
	return totalBytes;
	

//	return -1;
}
	
void ftdi_closeDevice() {
	FT_Close(ftHandle);
}