#include <ftdi.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <commands.h>
#include <errno.h>
#include <wireshark.h>
#include <sys/time.h>
#include <inttypes.h>

#define VERSION "1.0.2"
#define BUFSIZE 512

#define APP_SNIFFER 0
#define APP_ED 1

uint8_t  pktBuf[BUFSIZE];
int pktSize = 0;

uint8_t txBuf[BUFSIZE];
int txSize = 0;

uint8_t haveSTX = 0;

int device = 0;
int edFreq = -1;
int application = APP_SNIFFER;
char *outFileName = 0;
char helpNeeded = 0;
char *pipeName = 0;
int wireshark = 0;
FILE *outFile = 0;
FILE *pipeFile = 0;

void writePacketWireshark(uint8_t* buf, int size) {
	pcaprec_hdr_t hdr;

	struct timeval  tv1;	
	gettimeofday(&tv1, NULL);
	
	hdr.ts_sec = tv1.tv_sec;
	hdr.ts_usec = tv1.tv_usec;
	hdr.incl_len = hdr.orig_len = size;
	
	if (outFile != 0) {								
		fwrite(&hdr, sizeof(pcaprec_hdr_t), 1, outFile);		
		fwrite(buf, size, 1, outFile);
		fflush(outFile);
	} else {
//		fwrite(&hdr, sizeof(pcaprec_hdr_t), 1, stdout);		
//		fwrite(buf, size, 1, stdout);
//		fflush(stdout);
	}

	if (pipeName != 0) {
		fwrite(&hdr, sizeof(pcaprec_hdr_t), 1, pipeFile);
		fwrite(buf, size, 1, pipeFile);
		fflush(pipeFile);
	}	
}

void writePacket(FILE* file, uint8_t* buf, int size) {
	struct timeval  tv1;	
	gettimeofday(&tv1, NULL);

	fprintf(file, "%d.%d - ", tv1.tv_sec, tv1.tv_usec);
	for (int i = 0; i < size; i++) {
		if (i != 0)
			fprintf(file, " ");
		fprintf(file, "%02X", *(buf + i));
	}
	fprintf(file, "\n");
}

void resetDevice() {
	cmdbuf_t buf;
	ResetProcessorCmd_req* req = (StartApplicationCmd_req*)buf;
	req->m_ucCmdId = RESET_PROCESSOR_REQ;
	
	int st = serializeRequest(txBuf + txSize, BUFSIZE - txSize, (BasicCmd_Req*)buf);
	txSize += escapePacket(txBuf + txSize, st);
}

void setRadioConf(ConfRadio_req *req) {
	switch (application) {
	    case APP_SNIFFER:
	    {
		for (int i = 0; i < 16; i++) {
		    req->m_ucMode[i] = i;
		}
	    }
	    break;
	    case APP_ED:
	    {
		for (int i = 0; i < 16; i++) {
		    req->m_ucMode[i] = (i | (edFreq << 4));
		}
	    }
	    break;
	    default:
		printf("Unknown application %d, stopping...\n", application);
		exit(1);
	    break;
	}
}

void logEDPacket(EDPacket_resp *rsp) {
    printf("%" PRIu64 ", %d, %d\n", rsp->m_ullTimeStamp, rsp->m_ucChannel, rsp->m_ucRsl);
}

void processPacket(uint8_t* buf, int size) {

	size = unescapePacket(buf, size);
	cmdbuf_t cmdBuf;
	BasicCmd_Resp* rsp = (BasicCmd_Resp*)&cmdBuf[0];
	
	int sz = parseResponse(buf, size, &cmdBuf[0]);
	switch (rsp->m_ucCmdId) {
		case STATUS_RSP:
		{
			fprintf(stderr, "Starting sniffer...\n");
		
			cmdbuf_t buf;
			StartApplicationCmd_req* req = (StartApplicationCmd_req*)buf;
			req->m_ucCmdId = START_APPLICATION_REQ;
			
			int st = serializeRequest(txBuf + txSize, BUFSIZE - txSize, (BasicCmd_Req*)buf);	
			txSize += escapePacket(txBuf + txSize, st);			
		}
		break;		
		case START_APPLICATION_RSP:
		{
			cmdbuf_t buf;
			ConfRadio_req* req = (ConfRadio_req*)buf;
			req->m_ucCmdId = CONF_RADIO_REQ;

			setRadioConf(req);

			int st = serializeRequest(txBuf + txSize, BUFSIZE - txSize, (BasicCmd_Req*)buf);	
			txSize += escapePacket(txBuf + txSize, st);			
		}
		break;
		case RADIO_PACKET_RSP:
		{
			FILE *outf = stdout;
			if (outFile != 0) {				
				outf = outFile;
			}
			
			if (wireshark) {
				writePacketWireshark(buf + 1, size - 1);	// skip cmdId
			}
			else {
				writePacket(outf, buf + 1, size - 1);
				fflush(outf);
			}
		}
		break;
		case ED_PACKET_RSP:
		{
		    logEDPacket((EDPacket_resp*)rsp);
		}
		break;
		case CONF_RADIO_RSP:
		{
			cmdbuf_t buf;
			DebugInfo_req* req = (DebugInfo_req*)buf;
			req->m_ucCmdId = DEBUGINFO_REQ;

			int st = serializeRequest(txBuf + txSize, BUFSIZE - txSize, (BasicCmd_Req*)req);	
			txSize += escapePacket(txBuf + txSize, st);
		}
		break;
		case DEBUGINFO_RSP:
		{
			cmdbuf_t buf;
			StartLiveCapture_req *req = (StartLiveCapture_req*)buf;
			req->m_ucCmdId = START_LIVE_CAPTURE_REQ;
			req->m_ucMode = 1;//LIVE_CAPTURE_USB;

			int st = serializeRequest(txBuf + txSize, BUFSIZE - txSize, (BasicCmd_Req*)req);	
			txSize += escapePacket(txBuf + txSize, st);
		}
		break;
		case START_LIVE_CAPTURE_RSP:
		{
			// nothing to do, live capture started
		}
		break;
		case RESET_PROCESSOR_RSP:
		break;
		default:
		{
/*			cmdbuf_t buf;
			ResetProcessorCmd_req *req = (ResetProcessorCmd_req *)buf;
			req->m_ucCmdId = RESET_PROCESSOR_REQ;

			int st = serializeRequest(txBuf + txSize, BUFSIZE - txSize, (BasicCmd_Req*)req);	
			txSize += escapePacket(txBuf + txSize, st);
*/		}
			fprintf(stderr, "Received unexpected packet with cmdId=%d\n", rsp->m_ucCmdId);
		break;
	}
}

void writeWiresharkGeneralHeader() {
	pcap_hdr_t pcaphdr;
	pcaphdr.magic_number = 0xa1b2c3d4;
	pcaphdr.version_major = 2;
	pcaphdr.version_minor = 4;
	pcaphdr.thiszone = 0;
	pcaphdr.sigfigs = 0;
	pcaphdr.snaplen = 65535;
	pcaphdr.network = 230;

	if (outFile != 0) {								
		fwrite(&pcaphdr, sizeof(pcap_hdr_t), 1, outFile);		
	}
        else {
//		fwrite(&pcaphdr, sizeof(pcap_hdr_t), 1, stdout);		
	}
	if (pipeName != 0) {
		fwrite(&pcaphdr, sizeof(pcap_hdr_t), 1, pipeFile);		
	}
}


void runLoop() {

	if (wireshark) {
		writeWiresharkGeneralHeader();
	}

	while (1) {
	
	if (txSize > 0) {
		int res = ftdi_writeBytes(txBuf, txSize);
		if (res >= 0)
			memmove(txBuf, txBuf + res, res);
			txSize -= res;
	}
	
	int readBytes = ftdi_readBytes(pktBuf + pktSize, BUFSIZE - pktSize);
/*	if (readBytes > 0) {
	for (int i = 0; i < readBytes; i++) {
	    fprintf(stderr, "%02X", *(pktBuf + pktSize + i));
	    if (i < readBytes - 1)
		fprintf(stderr, " ");
	    else 
		fprintf(stderr, "\n");
	}
	}
*/	while (readBytes > 0) {
		if (!haveSTX) {
			int i = 0;
			for (; i < readBytes; i++) {
			
				if (pktBuf[pktSize + i] == STX) {
					haveSTX = 1;
					break;
				}
			}
			
			if (i > 0)
				memmove(pktBuf + pktSize, pktBuf + pktSize + i, readBytes - i);
				
			readBytes -= i;
			if (haveSTX) {
				pktSize++;
				readBytes--;
			}
		}
		if (haveSTX) {
			for (int i = 0; i < readBytes; i++)
			{
				if (pktBuf[pktSize] == ETX) {
					processPacket(pktBuf, pktSize + 1);
					memmove(pktBuf, pktBuf + pktSize, BUFSIZE - pktSize);
					haveSTX = 0;
					pktSize = 0;
//					readBytes --;
					break;
				}
				readBytes--;
				pktSize++;
			}
		}		
	}
	if (!haveSTX)
	    pktSize = 0;	// drop all remaining bytes
	
	}
}

void run_device(int index) {
	if (ftdi_openDevice(index) < 0) {
		return;
	}
		
	if (outFileName != 0) {
		if (wireshark) 
			outFile = fopen(outFileName, "wb+");
		else
			outFile = fopen(outFileName, "w+");
	}	

	if (pipeName != 0) {
		if (wireshark) 
			pipeFile = fopen(pipeName, "wb+");
		else
			pipeFile = fopen(pipeName, "w+");
	}	
	
	
	resetDevice();
		
	runLoop();
	
	ftdi_closeDevice();
}

void printhelp() {
    printf("Usage: ./adapter [-option]\n");
    printf("Options:\n");
    printf("\t\t-i\t\tSelect input device index. Default is 0.\n");
    printf("\t\t-o\t\tSelect output file name. Default is stdout.\n");
    printf("\t\t-p\t\tSelect pipe on which to send data. Default is null.\n");
    printf("\t\t-w\t\tPrepend Wireshark header to data. Default is off.\n");
    printf("\t\t-e <freq>\t\tStart the Energy Detect flow with a specified frequency:\n");
    printf("\t\t\t\t\t 0 for Continuous [not supported yet]\n");
    printf("\t\t\t\t\t 1 for 5 ms [not supported yet]\n");
    printf("\t\t\t\t\t 2 for 50 ms [not supported yet]\n");
    printf("\t\t\t\t\t 3 for 100 ms [not supported yet]\n");
    printf("\t\t\t\t\t 4 for 500 ms\n");
    printf("\t\t\t\t\t 5 for 1 s\n");
    printf("\t\t\t\t\t 6 for 2 s\n");
    printf("\t\t\t\t\t 7 for 5 s\n");
    printf("\t\t\t\t\t 8 for 10 s\n");
    printf("\t\t-help\t\tDisplays this help page.\n");
}

int main(int argc, char* argv[]) {
	
	printf("802.15.4 Site Analyzer Adapter for Linux version "); printf(VERSION);printf("\n");
	printf("(C)2016-2017 BeamLogic. All rights reserved.\n");
	for (int i = 1; i < argc; i++) {
		char * param = argv[i];
		if (strncmp("-o", param, 2) == 0) {
			if (i + 1 < argc) {
				outFileName = argv[++i]; 
			} else {
				printf("Illegal argument near -o. Please run with -help for help.\n");
				exit(1);
			}
		} else 
		if (strncmp("-w", param, 2) == 0) {
			wireshark = 1;
		} else 
		if (strncmp("-p", param, 2) == 0) {
			if (i + 1 < argc) {
				pipeName = argv[++i]; 
			} else {
				printf("Illegal argument near -p. Please run with -help for help.\n");
				exit(1);
			}
		} else 
		if (strncmp("-i", param, 2) == 0) {
			if (i + 1 < argc) {
				char *end;
				device = (int)strtol(argv[++i], &end, 10);
				if (errno != 0) {
					printf("Illegal argument near -i. Please run with -help for help.\n");
					exit(1);				
				}
			} else {
				printf("Illegal argument near -i. Please run with -help for help.\n");
				exit(1);
			}			
		} else 
		if (strncmp("-e", param, 2) == 0) {
			if (i + 1 < argc) {
				char *end;
				edFreq = (int)strtol(argv[++i], &end, 10);
				if (errno != 0 || (edFreq < 0) || (edFreq > 8)) {
					printf("Illegal argument near -e. Please run with -help for help.\n");
					exit(1);				
				}
				application = APP_ED;
			} else {
				printf("Illegal argument near -e. Please run with -help for help.\n");
				exit(1);
			}
		} else
		if (strncmp("-help", param, 5) == 0) {
		    printhelp();
		    exit(0);
		}
	}
	printf("Running with device %d.\n", device);
	run_device(device);

	return 0;
}
